@@ -24,8 +24,8 @@ By Dan Porter, PhD
 Diamond
 2017
 
-Version 2.7
-Last updated: 30/03/20
+Version 3.0
+Last updated: 27/05/20
 
 Version History:
 27/07/17 1.0    Version History started.
@@ -39,21 +39,26 @@ Version History:
 12/08/19 2.5    self.info outputs string, __repr__ methods added
 12/12/19 2.6    Added Symmetry.is_symmetric_reflection(ref1, ref2), added multiple scattering code
 30/03/20 2.7    Moved Multicrystal class to separate file, other minor tweaks
+19/04/20 2.8    Added update_cif and write_cif funcitons
+12/05/20 2.9    Updated Atom.from_cif to be more reliable
+27/05/20 3.0    Updated write_cif for magnetic moments - now writes simple mcif structures
 
 @author: DGPorter
 """
 
 import numpy as np
+from warnings import warn
 
 # Internal functions
 from . import functions_general as fg, functions_crystallography as fc
 from . import functions_crystallography as fc
 from .classes_properties import Properties
+#from .classes_orbitals import CrystalOrbitals
 from .classes_scattering import Scattering
 from .classes_multicrystal import MultiCrystal
 from .classes_plotting import Plotting, PlottingSuperstructure
 
-__version__ = '2.7'
+__version__ = '3.0'
 
 
 class Crystal:
@@ -71,6 +76,7 @@ class Crystal:
       xtl.Structure.uvw() >> give the full, unsymmetrised structure
       xtl.Scatter.hkl([1,0,0],8.00) >> prints the intensity and two-theta of this reflection at this energy
       xtl.Scatter.print_all_reflections(8.00) >> print all allowed reflections, with intensities, at this energy
+      xtl.write_cif('Diamond2.cif') >> write updated structure to file
     
     To create your own crystal (BCC example):
       xtl = Crystal()
@@ -100,6 +106,7 @@ class Crystal:
         self.Cell = Cell()
         self.Symmetry = Symmetry()
         self.Atoms = Atoms()
+        self.Structure = Atoms()
 
         # Get data from cif file
         if filename is not None:
@@ -124,6 +131,7 @@ class Crystal:
         self.Plot = Plotting(self)
         self.Scatter = Scattering(self)
         self.Properties = Properties(self)
+        #self.Orbitals = CrystalOrbitals(self) # slows down cif parsing a lot!
         # self.Fdmnes = Fdmnes(self)
 
     def generate_structure(self):
@@ -164,6 +172,46 @@ class Crystal:
 
         self.Structure = Atoms(u, v, w, new_type, new_label, new_occupancy, new_uiso, new_mxmymz)
 
+    def update_cif(self, cifvals=None):
+        """
+        Update self.cif dict with current values
+        :param cifvals: cif dict from readcif (None to use self.cif)
+        :return: cifvals
+        """
+
+        if cifvals is None:
+            cifvals = self.cif
+
+        # Update name
+        cifvals['_pd_phase_name'] = self.name
+        cifvals = self.Cell.update_cif(cifvals)
+        cifvals = self.Symmetry.update_cif(cifvals)
+        cifvals = self.Atoms.update_cif(cifvals)
+        cifvals = self.Properties.update_cif(cifvals)
+
+        self.cif = cifvals
+        return cifvals
+
+    def write_cif(self, filename=None, comments=None):
+        """
+        Write crystal structure to CIF (Crystallographic Information File)
+         Only basic information is saved to the file, but enough to open in VESTA etc.
+         If magnetic ions are defined, a magnetic cif (*.mcif) will be produce
+        :param filename: name to write too, if None, use writes to self.name (.cif/.mcif)
+        :param comments: str comments to add to file header
+        :return: None
+        """
+
+        cifvals = self.update_cif()
+
+        if filename is None:
+            filename = '%s' % fg.saveable(self.name)
+
+        if self.Atoms.ismagnetic():
+            fc.write_mcif(cifvals, filename, comments)
+        else:
+            fc.write_cif(cifvals, filename, comments)
+
     def new_cell(self, latt=[1.0]):
         """
         Replace the lattice parameters
@@ -192,8 +240,12 @@ class Crystal:
 
     def generate_lattice(self, U=1, V=1, W=0):
         """
-        Generate lattice of a cell
-         latt = genlatt(crys,U=1,V=1,W=0)
+        Generate a repeated lattice of the crystal structure
+            latt = xtl.generate_lattice(2,0,0)
+        :param U: Repeat of the cell along the a axis
+        :param V: Repeat of the cell along the b axis
+        :param W: Repeat of the cell along the c axis
+        :return: Crystal object
         """
 
         uvw, type, label, occ, uiso, mxmymz = self.Structure.generate_lattice(U, V, W, centred=False)
@@ -274,8 +326,9 @@ class Crystal:
         :return: str
         """
         out = '\n###########################################\n'
-        out += '{}'.format(self.name)
-        out += '\nFormula: {}\n'.format(self.Properties.molname())
+        out += '{}\n'.format(self.name)
+        out += 'Formula: {}\n'.format(self.Properties.molname())
+        out += 'Magnetic: {}\n'.format(self.Structure.ismagnetic())
         out += self.Cell.info()
         out += '\nDensity: %6.3f g/cm\n' % self.Properties.density()
         out += self.Structure.info()
@@ -284,6 +337,10 @@ class Crystal:
         return out
 
     def __repr__(self):
+        if self.Structure.ismagnetic():
+            nmom = np.sum(np.any(np.abs(self.Structure.mxmymz()) > 0, axis=1))
+            fmt = "%s with %d atomic positions (%d magnetic), %d symmetries"
+            return fmt % (self.name, len(self.Atoms.type), nmom, len(self.Symmetry.symmetry_operations))
         fmt = "%s with %d atomic positions, %d symmetries"
         return fmt % (self.name, len(self.Atoms.type), len(self.Symmetry.symmetry_operations))
 
@@ -299,6 +356,14 @@ class Cell:
         UC.tth([0,0,12],energy_kev=8.0) # Calculate the two-theta of a reflection
         UC.lp() # Returns the current lattice parameters
     """
+    _required_cif = [
+        "_cell_length_a",
+        "_cell_length_b",
+        "_cell_length_c",
+        "_cell_angle_alpha",
+        "_cell_angle_beta",
+        "_cell_angle_gamma"
+    ]
 
     def __init__(self, a=1.0, b=1.0, c=1.0, alpha=90., beta=90.0, gamma=90.0):
         self.latt([a, b, c, alpha, beta, gamma])
@@ -334,10 +399,20 @@ class Cell:
 
     def fromcif(self, cifvals):
         """
-        Import lattice parameters from a cif dictionary"
+        Import lattice parameters from a cif dictionary
+        Required CIF keys:
+            _cell_length_a
+            _cell_length_b
+            _cell_length_c
+            _cell_angle_alpha
+            _cell_angle_beta
+            _cell_angle_gamma
         :param cifvals: dict from readcif
         :return: None
         """
+        if not fc.cif_check(cifvals, self._required_cif):
+            warn('Lattice parameters cannot be read from cif')
+            return
 
         a, da = fg.readstfm(cifvals['_cell_length_a'])
         b, db = fg.readstfm(cifvals['_cell_length_b'])
@@ -348,6 +423,23 @@ class Cell:
 
         self.latt([a, b, c, alpha, beta, gamma])
 
+    def update_cif(self, cifvals):
+        """
+        Update cif dict with current values
+        :param cifvals: dict from readcif
+        :return: cifvals
+        """
+
+        cifvals['_cell_length_a'] = '%1.6f' % self.a
+        cifvals['_cell_length_b'] = '%1.6f' % self.b
+        cifvals['_cell_length_c'] = '%1.6f' % self.c
+        cifvals['_cell_angle_alpha'] = '%1.4g' % self.alpha
+        cifvals['_cell_angle_beta'] = '%1.4g' % self.beta
+        cifvals['_cell_angle_gamma'] = '%1.4g' % self.gamma
+
+        cifvals['_cell_volume'] = '%1.4f' % self.volume()
+        return cifvals
+
     def lp(self):
         """
         Returns the lattice parameters
@@ -656,6 +748,13 @@ class Atoms:
     _default_atom = 'Fe'
     _default_uiso = 1.0 / (8 * np.pi ** 2)  # B=1
 
+    _required_cif = [
+        "_atom_site_label",
+        "_atom_site_fract_x",
+        "_atom_site_fract_y",
+        "_atom_site_fract_z",
+    ]
+
     def __init__(self, u=[0], v=[0], w=[0], type=None,
                  label=None, occupancy=None, uiso=None, mxmymz=None):
         " Initialisation, defines Atoms defaults"
@@ -698,15 +797,32 @@ class Atoms:
 
     def __call__(self, u=[0], v=[0], w=[0], type=None,
                  label=None, occupancy=None, uiso=None, mxmymz=None):
-        "Re-initialises the class, generating new atomic positions"
+        """Re-initialises the class, generating new atomic positions"""
         self.__init__(u, v, w, type, label, occupancy, uiso, mxmymz=None)
 
     def fromcif(self, cifvals):
         """
         Import atom parameters from a cif dictionary
+        Required cif keys:
+            _atom_site_label
+            _atom_site_fract_x
+            _atom_site_fract_y
+            _atom_site_fract_z
+        Optional cif keys:
+            _atom_site_type_symbol
+            _atom_site_U_iso_or_equiv
+            _atom_site_B_iso_or_equiv
+            _atom_site_occupancy
+            _atom_site_moment_label
+            _atom_site_moment_crystalaxis_x
+            _atom_site_moment_crystalaxis_y
+            _atom_site_moment_crystalaxis_z
         :param cifvals: dict
         :return: none
         """
+        if not fc.cif_check(cifvals, self._required_cif):
+            warn('Atom site parameters cannot be read from cif')
+            return
 
         keys = cifvals.keys()
 
@@ -714,11 +830,21 @@ class Atoms:
         label = cifvals['_atom_site_label']
 
         if '_atom_site_type_symbol' in keys:
-            element = [x.strip('+-0123456789') for x in cifvals['_atom_site_type_symbol']]
+            element = [fc.str2element(x) for x in cifvals['_atom_site_type_symbol']]
         else:
-            element = [x.strip('+-0123456789') for x in cifvals['_atom_site_label']]
+            element = [fc.str2element(x) for x in cifvals['_atom_site_label']]
 
-        # Get other properties
+        while False in element:
+            fidx = element.index(False)
+            warn('%s is not a valid element, replacing with H' % cifvals['_atom_site_label'][fidx])
+            element[fidx] = 'H'
+
+        # Replace Deuterium with Hydrogen
+        if 'D' in element:
+            warn('Replacing Deuterium ions with Hydrogen')
+            element = ['H' if el == 'D' else el for el in element]
+
+        # Thermal parameters
         if '_atom_site_U_iso_or_equiv' in keys:
             uiso = np.array([fg.readstfm(x)[0] for x in cifvals['_atom_site_U_iso_or_equiv']])
         elif '_atom_site_B_iso_or_equiv' in keys:
@@ -726,15 +852,16 @@ class Atoms:
             uiso = fc.biso2uiso(biso)
         else:
             uiso = np.zeros(len(element))
+        # Occupancy
         if '_atom_site_occupancy' in keys:
             occ = np.array([fg.readstfm(x)[0] for x in cifvals['_atom_site_occupancy']])
         else:
             occ = np.ones(len(element))
 
         # Get coordinates
-        u = np.array([fg.readstfm(x)[0] for x in cifvals['_atom_site_fract_x']])
-        v = np.array([fg.readstfm(x)[0] for x in cifvals['_atom_site_fract_y']])
-        w = np.array([fg.readstfm(x)[0] for x in cifvals['_atom_site_fract_z']])
+        u = np.array([fg.readstfm(x)[0] for x in cifvals['_atom_site_fract_x']], dtype=np.float)
+        v = np.array([fg.readstfm(x)[0] for x in cifvals['_atom_site_fract_y']], dtype=np.float)
+        w = np.array([fg.readstfm(x)[0] for x in cifvals['_atom_site_fract_z']], dtype=np.float)
 
         # Get magnetic vectors
         mx = np.zeros(len(u))
@@ -763,6 +890,44 @@ class Atoms:
         self.my = my
         self.mz = mz
 
+    def update_cif(self, cifvals):
+        """
+        Update cif dict with stored values
+        :param cifvals: cif dict from readcif
+        :return: cifvals
+        """
+
+        keys = cifvals.keys()
+
+        cifvals['_atom_site_label'] = self.label
+        cifvals['_atom_site_type_symbol'] = self.type
+        cifvals['_atom_site_U_iso_or_equiv'] = self.uiso
+        cifvals['_atom_site_occupancy'] = self.occupancy
+        cifvals['_atom_site_fract_x'] = self.u
+        cifvals['_atom_site_fract_y'] = self.v
+        cifvals['_atom_site_fract_z'] = self.w
+
+        # Magnetic moments
+        mag_label = []
+        mag_x = []
+        mag_y = []
+        mag_z = []
+        mag_sym = []
+        for n in range(len(self.label)):
+            if self.mx[n]**2 + self.my[n]**2 + self.mz[n]**2 > 0.01:
+                mag_label += [self.label[n]]
+                mag_x += [self.mx[n]]
+                mag_y += [self.my[n]]
+                mag_z += [self.mz[n]]
+                mag_sym += ['mx,my,mz']
+
+        cifvals['_atom_site_moment.label'] = mag_label
+        cifvals['_atom_site_moment.crystalaxis_x'] = mag_x
+        cifvals['_atom_site_moment.crystalaxis_y'] = mag_y
+        cifvals['_atom_site_moment.crystalaxis_z'] = mag_z
+        #cifvals['_atom_site_moment.symmform'] = mag_sym
+        return cifvals
+
     def changeatom(self, idx, u=None, v=None, w=None, type=None,
                    label=None, occupancy=None, uiso=None, mxmymz=None):
         """
@@ -959,6 +1124,10 @@ class Atoms:
         """
         return np.asarray([self.mx, self.my, self.mz], dtype=np.float).T
 
+    def ismagnetic(self):
+        """ Returns True if any ions have magnetic moments assigned"""
+        return np.any(np.abs(self.mxmymz())>0)
+
     def get(self):
         """
         Returns the structure arrays
@@ -1043,17 +1212,22 @@ class Atoms:
 
         out = ''
         if np.any(fg.mag(self.mxmymz()) > 0):
-            out += '  n Atom  u       v       w        Occ  Uiso      mx      my      mz      \n'
-            fmt = '%3.0f %4s %7.4f %7.4f %7.4f   %4.2f %6.4f   %7.4f %7.4f %7.4f\n'
+            out += '  n Label Atom  u       v       w        Occ  Uiso      mx      my      mz      \n'
+            fmt = '%3.0f %5s %4s %7.4f %7.4f %7.4f   %4.2f %6.4f   %7.4f %7.4f %7.4f\n'
             for n in disprange:
                 out += fmt % (
-                    n, self.type[n], self.u[n], self.v[n], self.w[n], self.occupancy[n], self.uiso[n], self.mx[n],
-                    self.my[n], self.mz[n])
+                    n, self.label[n], self.type[n], self.u[n], self.v[n], self.w[n],
+                    self.occupancy[n], self.uiso[n],
+                    self.mx[n], self.my[n], self.mz[n]
+                )
         else:
-            out += '  n Atom  u       v       w        Occ  Uiso\n'
-            fmt = '%3.0f %4s %7.4f %7.4f %7.4f   %4.2f %6.4f\n'
+            out += '  n Label Atom  u       v       w        Occ  Uiso\n'
+            fmt = '%3.0f %5s %4s %7.4f %7.4f %7.4f   %4.2f %6.4f\n'
             for n in disprange:
-                out += fmt % (n, self.type[n], self.u[n], self.v[n], self.w[n], self.occupancy[n], self.uiso[n])
+                out += fmt % (
+                    n, self.label[n], self.type[n], self.u[n], self.v[n], self.w[n],
+                    self.occupancy[n], self.uiso[n]
+                )
         return out
 
     def __repr__(self):
@@ -1069,11 +1243,12 @@ class Symmetry:
     spacegroup_number = 1
     symmetry_operations = ['x,y,z']
     symmetry_operations_magnetic = ['x,y,z']
+    #symmetry_operations_time = [1]
 
     # symmetry_operations_time = [1]
 
     def __init__(self, symmetry_operations=None, symmetry_operations_magnetic=None):
-        "Initialises the symmetry group"
+        """Initialises the symmetry group"""
 
         if symmetry_operations is not None:
             self.addsym(symmetry_operations, symmetry_operations_magnetic)
@@ -1083,6 +1258,21 @@ class Symmetry:
     def fromcif(self, cifvals):
         """
         Import symmetry information from a cif dictionary
+        Required cif keys:
+            None
+        Optional cif keys:
+            _symmetry_equiv_pos_as_xyz
+            _space_group_symop_operation_xyz
+            _space_group_symop_magn_operation_xyz
+            _space_group_symop_magn_operation_mxmymz
+            _space_group_symop_magn_centering_xyz
+            _space_group_symop_magn_centering_mxmymz
+            _symmetry_space_group_name_H-M
+            _space_group_name_H-M_alt
+            _space_group_magn_name_BNS
+            _symmetry_Int_Tables_number
+            _space_group_IT_number
+            _space_group_magn_number_BNS
         :param cifvals: dict of values from cif
         :return:
         """
@@ -1090,26 +1280,22 @@ class Symmetry:
         keys = cifvals.keys()
 
         # Get symmetry operations
-        ops = ['+1/2', '+1/3', '+2/3', '+1/4', '+3/4', '+1/6', '+5/6', \
-               '1/2+', '1/3+', '2/3+', '1/4+', '3/4+', '1/6+', '5/6+', \
-               '-1/2', '-1/3', '-2/3', '-1/4', '-3/4', '-1/6', '-5/6', \
-               '1/2-', '1/3-', '2/3-', '1/4-', '3/4-', '1/6-', '5/6-']  # replace these in magnetic symmetry operations
         if '_symmetry_equiv_pos_as_xyz' in keys:
             symops = cifvals['_symmetry_equiv_pos_as_xyz']
             symcen = ['x,y,z']
 
             # add magnetic symmetries (symops without translation)
-            symops_mag = [fg.multi_replace(sp, ops, '') for sp in symops]
-            cenops_mag = [fg.multi_replace(sp, ops, '') for sp in symcen]
-            # self.symmetry_operations_time = [1]*len(symops)
-            # self.centring_operations_time = [1]
+            symops_mag = fc.symmetry_ops2magnetic(symops)
+            cenops_mag = fc.symmetry_ops2magnetic(symcen)
+            #self.symmetry_operations_time = [1]*len(symops)
+            #self.centring_operations_time = [1]
         elif '_space_group_symop_operation_xyz' in keys:
             symops = cifvals['_space_group_symop_operation_xyz']
             symcen = ['x,y,z']
 
             # add magnetic symmetries (symops without translation)
-            symops_mag = [fg.multi_replace(sp, ops, '') for sp in symops]
-            cenops_mag = [fg.multi_replace(sp, ops, '') for sp in symcen]
+            symops_mag = fc.symmetry_ops2magnetic(symops)
+            cenops_mag = fc.symmetry_ops2magnetic(symcen)
             # self.symmetry_operations_time = [1]*len(symops)
             # self.centring_operations_time = [1]
         elif '_space_group_symop_magn_operation_xyz' in keys:
@@ -1120,7 +1306,7 @@ class Symmetry:
             if '_space_group_symop_magn_operation_mxmymz' in keys:
                 symmag = cifvals['_space_group_symop_magn_operation_mxmymz']  # mx,my,mz
             else:
-                symmag = [fg.multi_replace(sp, ops, '') for sp in symops]
+                symmag = fc.symmetry_ops2magnetic(symops)
                 # invert magnetic symmetry when time odd
                 for n in range(len(symmag)):
                     if symtim[n] < 0:
@@ -1133,7 +1319,7 @@ class Symmetry:
             if '_space_group_symop_magn_centering_mxmymz' in keys:
                 symcenmag = cifvals['_space_group_symop_magn_centering_mxmymz']  # mx,my,mz
             else:
-                symcenmag = [fg.multi_replace(sp, ops, '') for sp in symcen]
+                symcenmag = fc.symmetry_ops2magnetic(symcen)
                 # invert magnetic symmetry when time odd
                 for n in range(len(symcenmag)):
                     if symcentim[n] < 0:
@@ -1184,6 +1370,73 @@ class Symmetry:
 
         self.generate_matrices()
 
+    def update_cif(self, cifvals):
+        """
+        Update cifvals dict with current symmetry operations
+        :param cifvals: cif dict from functions_crystallography.readcif
+        :return: cifvals
+        """
+        keys = cifvals.keys()
+
+        cifvals['_symmetry_equiv_pos_as_xyz'] = self.symmetry_operations
+        cifvals['_space_group_symop_operation_xyz'] = self.symmetry_operations
+
+        # newer versions of mcif don't use magn_operation.mxmymz but use symform in atom spec
+        # . may be in the wrong place for some program
+        magsym = [
+            '%s' % op.replace('x', 'mx').replace('y', 'my').replace('z', 'mz')
+            for op in self.symmetry_operations_magnetic
+        ]
+        cifvals['_space_group_symop_magn_operation.id'] = range(1, len(self.symmetry_operations)+1)
+        cifvals['_space_group_symop_magn_operation.xyz'] = ['%s, +1' % op for op in self.symmetry_operations]
+        cifvals['_space_group_symop_magn_operation.mxmymz'] = magsym
+        cifvals['_space_group_symop_magn_centering.id'] = ['1']
+        cifvals['_space_group_symop_magn_centering.xyz'] = ['x,y,z,+1']
+        cifvals['_space_group_symop_magn_centering.mxmymz'] = ['mx,my,mz']
+
+        # Get space group
+        cifvals['_symmetry_space_group_name_H-M'] = self.spacegroup
+        cifvals['_space_group_name_H-M_alt'] = self.spacegroup
+        cifvals['_space_group_magn.name_BNS'] = self.spacegroup
+
+        cifvals['_symmetry_Int_Tables_number'] = self.spacegroup_number
+        cifvals['_space_group_IT_number'] = self.spacegroup_number
+        cifvals['_space_group_magn.number_BNS'] = self.spacegroup_number
+        return cifvals
+
+    def load_spacegroup(self, sg_number):
+        """
+        Load symmetry operations from a spacegroup from the International Tables of Crystallogrphy
+        See functions_crystallography.spacegroup for more details
+        :param sg_number: space group number (1-230)
+        :return: None
+        """
+        spacegroup = fc.spacegroup(sg_number)
+        self.spacegroup_number = int(spacegroup['space group number'])
+        self.spacegroup = spacegroup['space group name']
+
+        symops = spacegroup['general positions']
+        self.symmetry_operations = symops
+        self.symmetry_operations_magnetic = fc.symmetry_ops2magnetic(symops)
+
+    def load_magnetic_spacegroup(self, msg_number):
+        """
+        Load symmetry operations from a magnetic spacegroup from Bilbao crystallographic server
+        Replaces the current symmetry operators and the magnetic symmetry operators.
+        See functions_crystallography.spacegroup_magnetic for more details
+        :param msg_number: magnetic space group number e.g. 61.433
+        :return:
+        """
+
+        maggroup = fc.spacegroup_magnetic(msg_number)
+        self.spacegroup_number = maggroup['space group number']
+        self.spacegroup = maggroup['space group name']
+        symops = maggroup['positions general']
+        symmag = maggroup['positions magnetic']
+        symmag = [op.replace('m', '') for op in symmag]
+        self.symmetry_operations = symops
+        self.symmetry_operations_magnetic = symmag
+
     def changesym(self, idx, operation):
         """
         Change a symmetry operation
@@ -1249,6 +1502,26 @@ class Symmetry:
 
         self.symmetry_matrices = fc.gen_sym_mat(self.symmetry_operations)
 
+    def print_subgroups(self, sg_number=None):
+        """
+        Return str of subgroups of this spacegroup
+        :param sg_number: spacegroup number, None to use current one
+        :return: str
+        """
+        if sg_number is None:
+            sg_number = self.spacegroup_number
+        return fc.spacegroup_subgroups_list(sg_number)
+
+    def print_magnetic_spacegroups(self, sg_number=None):
+        """
+        Return str of available magnetic spacegroups for this spacegroup
+        :param sg_number: spacegroup number, None to use current one
+        :return: str
+        """
+        if sg_number is None:
+            sg_number = self.spacegroup_number
+        return fc.spacegroup_magnetic_list(sg_number)
+
     def symmetric_coordinates(self, UVW, MXMYMZ=None, remove_identical=True):
         """
         Returns array of symmetric coordinates
@@ -1642,8 +1915,8 @@ class Superstructure(Crystal):
     superstructure Crystal class
     """
 
-    Parent = Crystal()
-    P = [[1, 0, 0], [0, 1, 0], [0, 0, 1]]
+    #Parent = Crystal()
+    #P = [[1, 0, 0], [0, 1, 0], [0, 0, 1]]
 
     def __init__(self, Parent, P):
         """Initialise"""
