@@ -11,8 +11,8 @@ Usage:
     OR
     - from Dans_Diffraction import functions_crystallography as fc
 
-Version 2.8
-Last updated: 03/04/20
+Version 3.1.1
+Last updated: 26/05/20
 
 Version History:
 09/07/15 0.1    Version History started.
@@ -26,27 +26,88 @@ Version History:
 06/03/19 2.6    Added print_atom_properties
 14/08/19 2.7    Added new Dans Element Properties file with extra comment line, new functions added
 03/04/20 2.8    Updated attenuation to work with arrays of elements
+19/04/20 2.9    Added write_cif, made alterations to readcif for speed and readability, added spacegroup
+01/05/20 3.0    Updated atom_properties, now have atomic properties above 92 with warning. Some changes to readcif.
+05/05/20 3.0.1  Further changes to readcif. Changed method of symmetry_ops2magnetic. Added str2element
+12/05/20 3.1.0  More readcif changes, added atomic_scattering_factor, element_charge_string, split_compound
+26/05/20 3.1.1  Updated magnetic space groups, added magnetic positions (was only generators), added write_mcif
+
+Acknoledgements:
+    April 2020  Thanks to ChunHai Wang for helpful suggestions in readcif!
 
 @author: DGPorter
 """
 
 import sys, os, re
+import json
 import numpy as np
+from warnings import warn
 
 from . import functions_general as fg
 
-__version__ = '2.7'
+__version__ = '3.1.1'
 
 # File directory - location of "Dans Element Properties.txt"
 datadir = os.path.abspath(os.path.dirname(__file__))  # same directory as this file
 datadir = os.path.join(datadir, 'data')
+ATOMFILE = os.path.join(datadir, 'Dans Element Properties.txt')
+
+# List of Elements in order sorted by length of name
+ELEMENT_LIST = [
+    'Zr', 'Mo', 'Es', 'Eu', 'Fe', 'Fl', 'Fm', 'Fr', 'Ga', 'Gd', 'Ge',
+    'He', 'Hf', 'Hg', 'Ho', 'Hs', 'In', 'Ir', 'Kr', 'La', 'Li', 'Lr',
+    'Lu', 'Lv', 'Mc', 'Zn', 'Mg', 'Er', 'Dy', 'Ds', 'Bk', 'Ag', 'Al',
+    'Am', 'Ar', 'As', 'At', 'Au', 'Ba', 'Be', 'Bh', 'Bi', 'Br', 'Db',
+    'Ca', 'Cd', 'Ce', 'Cf', 'Cl', 'Cm', 'Cn', 'Co', 'Cr', 'Cs', 'Cu',
+    'Mn', 'Md', 'Mt', 'Rb', 'Rf', 'Rh', 'Rn', 'Ru', 'Sb', 'Sc', 'Se',
+    'Sg', 'Si', 'Sm', 'Sn', 'Sr', 'Ta', 'Tb', 'Tc', 'Te', 'Th', 'Ti',
+    'Tl', 'Tm', 'Ts', 'Xe', 'Yb', 'Re', 'Rg', 'Ac', 'Pb', 'Nh', 'Ni',
+    'No', 'Np', 'Nd', 'Ra', 'Og', 'Os', 'Pa', 'Pd', 'Nb', 'Pm', 'Po',
+    'Pr', 'Pt', 'Na', 'Pu', 'Ne', 'B', 'W', 'V', 'Y', 'U', 'F', 'K',
+    'C', 'I', 'P', 'H', 'S', 'N', 'O',
+    'zr', 'mo', 'es', 'eu', 'fe', 'fl', 'fm', 'fr', 'ga', 'gd', 'ge',
+    'he', 'hf', 'hg', 'ho', 'hs', 'in', 'ir', 'kr', 'la', 'li', 'lr',
+    'lu', 'lv', 'mc', 'zn', 'mg', 'er', 'dy', 'ds', 'bk', 'ag', 'al',
+    'am', 'ar', 'as', 'at', 'au', 'ba', 'be', 'bh', 'bi', 'br', 'db',
+    'ca', 'cd', 'ce', 'cf', 'cl', 'cm', 'cn', 'co', 'cr', 'cs', 'cu',
+    'mn', 'md', 'mt', 'rb', 'rf', 'rh', 'rn', 'ru', 'sb', 'sc', 'se',
+    'sg', 'si', 'sm', 'sn', 'sr', 'ta', 'tb', 'tc', 'te', 'th', 'ti',
+    'tl', 'tm', 'ts', 'xe', 'yb', 're', 'rg', 'ac', 'pb', 'nh', 'ni',
+    'no', 'np', 'nd', 'ra', 'og', 'os', 'pa', 'pd', 'nb', 'pm', 'po',
+    'pr', 'pt', 'na', 'pu', 'ne', 'b', 'w', 'v', 'y', 'u', 'f', 'k',
+    'c', 'i', 'p', 'h', 's', 'n', 'o',
+    'D', 'd', # add Deuterium
+]
+element_regex = re.compile('|'.join(ELEMENT_LIST))
+
+# Symmetry translations (remove these to turn symmetry operations into magnetic one)
+TRANSLATIONS = [
+    '+1/2', '+1/3', '+2/3', '+1/4', '+3/4', '+1/6', '+5/6', '+5/4',
+    '1/2+', '1/3+', '2/3+', '1/4+', '3/4+', '1/6+', '5/6+', '5/4+',
+    '-1/2', '-1/3', '-2/3', '-1/4', '-3/4', '-1/6', '-5/6', '-5/4',
+    '1/2-', '1/3-', '2/3-', '1/4-', '3/4-', '1/6-', '5/6-', '5/4-',
+]
+
+# Required CIF keys, must be available and not '?'
+CIF_REQUIRES = [
+    "_cell_length_a",
+    "_cell_length_b",
+    "_cell_length_c",
+    "_cell_angle_alpha",
+    "_cell_angle_beta",
+    "_cell_angle_gamma",
+    "_atom_site_label",
+    "_atom_site_fract_x",
+    "_atom_site_fract_y",
+    "_atom_site_fract_z",
+]
 
 
 def getenergy():
     return 8.048  # Cu Kalpha energy, keV
 
 
-'--------------Functions to Load crystal Structures----------------------'
+'--------------Functions to Read & Write CIF files----------------------'
 
 
 def readcif(filename=None, debug=False):
@@ -104,7 +165,7 @@ def readcif(filename=None, debug=False):
     n = 0
     while n < len(lines):
         # Convert line to columns
-        vals = lines[n].split()
+        vals = lines[n].strip().split()
 
         # skip empty lines
         if len(vals) == 0:
@@ -113,24 +174,21 @@ def readcif(filename=None, debug=False):
 
         # Search for stored value lines
         if vals[0][0] == '_':
-            if len(vals) > 2 and ('\'' in vals[1] or '"' in vals[1]):
-                # catch strings with spaces
-                cifvals[vals[0]] = ' '.join(vals[1:]).strip('\'"')
-            elif len(vals) == 1 and ';' in lines[n + 1]:
-                # Catch multi-line arguments
-                n += 1
-                cifvals[vals[0]] = [lines[n].strip().replace(';', '')]
-                n += 1
-                while ';' not in lines[n]:
-                    cifvals[vals[0]] += [lines[n].strip()]
+            if len(vals) == 1:
+                # Record next lines that are not keys as string
+                if lines[n+1][0] == ';': n += 1
+                strarg = []
+                while n+1 < len(lines) and (len(lines[n+1]) == 0 or lines[n+1][0].strip() not in ['_', ';']):
+                    strarg += [lines[n+1].strip('\'"')]
                     n += 1
-                cifvals[vals[0]] = ' '.join(cifvals[vals[0]])
-            elif len(vals) > 1:
-                cifvals[vals[0]] = vals[1]
+                cifvals[vals[0]] = '\n'.join(strarg)
+                chk = 'a'
+            else:
+                cifvals[vals[0]] = ' '.join(vals[1:]).strip(' \'"\n')
+                chk = 'b'
             n += 1
-
             if debug:
-                print('%5d %s = %s' % (n, vals[0], cifvals[vals[0]]))
+                print('%5d %s %s = %s' % (n, chk, vals[0], cifvals[vals[0]]))
             continue
 
         # Search for loops
@@ -139,24 +197,24 @@ def readcif(filename=None, debug=False):
             loopvals = []
             # Step 1: Assign loop columns
             # looped columns are given by "_column_name"
-            while n < len(lines) and len(lines[n].strip()) > 0 and lines[n].strip()[
-                0] == '_':  # 21/5/18 split()[0][0] == '_':
+            while n < len(lines) and len(lines[n].strip()) > 0 and lines[n].strip()[0] == '_':
                 loopvals += [lines[n].split()[0]]
                 cifvals[loopvals[-1]] = []
                 n += 1
 
-            # Skip blank line after column names, e.g. 1000175.cif 21/5/18
-            # if len(lines[n].strip()) == 0: n += 1
-
             # Step 2: Assign data to columns
             # loops until line has less segments than columns
-            while n < len(lines) and len(lines[n].split()) >= len(loopvals):
-                cols = lines[n].split()
-                cols = cols[:len(loopvals) - 1] + [
-                    ''.join(cols[len(loopvals) - 1:])]  # fixes error on symmetry arguments having spaces
+            while n < len(lines):
+                #cols = lines[n].split()
+                # this fixes error on symmetry arguments having spaces
+                # this will only work if the last argument in the loop is split by spaces (in quotes)
+                #cols = cols[:len(loopvals) - 1] + [''.join(cols[len(loopvals) - 1:])]
+                cols = [col for col in re.split("( |\\\".*?\\\"|'.*?')", lines[n]) if col.strip()]
+                if len(cols) != len(loopvals): break
                 if cols[0][0] == '_' or cols[0] == 'loop_': break  # catches error if loop is only 1 iteration
+                if cols[0][0] == '#': n += 1; continue  # catches comented out lines
                 if len(loopvals) == 1:
-                    cifvals[loopvals[0]] += [lines[n].strip()]
+                    cifvals[loopvals[0]] += [lines[n].strip(' \"\'\n')]
                 else:
                     for c, ll in enumerate(loopvals):
                         cifvals[ll] += [cols[c]]
@@ -164,11 +222,13 @@ def readcif(filename=None, debug=False):
 
             if debug:
                 for ll in loopvals:
-                    print('%5d %s = %s' % (n, ll, str(cifvals[ll])))
+                    print('%5d L %s = %s' % (n, ll, str(cifvals[ll])))
             continue
 
         else:
             # Skip anything else
+            if debug:
+                print('%5d SKIPPED: %s' % (n, lines[n]))
             n += 1
 
     # Replace '.' in keys - fix bug from isodistort cif files
@@ -181,6 +241,29 @@ def readcif(filename=None, debug=False):
     return cifvals
 
 
+def cif_check(cifvals, required_keys=None, bad_value='?'):
+    """
+    Returns True if all basic required cif parameter are available and real.
+    E.G.:
+        cifvals = readcif(file.cif)
+        if cif_check(cifvals):
+            print('File OK')
+    :param cifvals: dict of cif keys form readcif
+    :param required_keys: list of key strings, or None for default
+    :param bad_value: if this value is in the cif key item, return False
+    :return: bool
+    """
+    if required_keys is None:
+        required_keys = CIF_REQUIRES
+    keys = cifvals.keys()
+    for key in required_keys:
+        if key not in keys:
+            return False
+        if bad_value in cifvals[key]:
+            return False
+    return True
+
+
 def cif2dict(cifvals):
     """
     From a dict of key:value pairs generated from a *.cif file (using readcif),
@@ -425,6 +508,223 @@ def cif2dict(cifvals):
     return crys
 
 
+def write_cif(cifvals, filename=None, comments=None):
+    """
+    Write .cif file with values from a cif dict,
+    Only basic items are saved, rather than returning the orginal file
+    :param cifvals: dict from readcif
+    :param filename: filename to write (use None to return string)
+    :param comments: str comments to write to the file top matter
+    :return: None
+    """
+
+    keys = cifvals.keys()
+
+    def cif_value(name):
+        if name in keys:
+            return '%-40s %-12s\n' % (name, cifvals[name])
+        else:
+            print('%s not in cif dict' % name)
+            return '%-40s %-12s\n' % (name, '?')
+
+    def cif_loop(names):
+        names = [name for name in names if name in keys]
+        if len(names) == 0:
+            print('Loop Items not in cif dict')
+            return ''
+        out = 'loop_\n'
+        out += ''.join(['%s\n' % name for name in names])
+        vals = [cifvals[name] for name in names]
+        for val_line in zip(*vals):
+            out += ' '.join(['%-12s' % val for val in val_line])
+            out += '\n'
+        return out
+
+    # Top Matter
+    c = '#----------------------------------------------------------------------\n'
+    c += '#   Crystal Structure: %s\n' % (cifvals['FileTitle'] if 'FileTitle' in keys else '')
+    c += '#----------------------------------------------------------------------\n'
+    c += '# CIF created in Dans_Diffraction\n'
+    c += '# Original cif:\n# %s\n' % (cifvals['Filename'] if 'Filename' in keys else 'None')
+
+    # Comments
+    c += '# Comments:\n'
+    if comments:
+        comments = comments.split('\n')
+        c += ''.join(['# %s\n' % comment for comment in comments])
+
+    # Crystal Data
+    c += '\ndata_WRITECIF\n'
+    c += cif_value('_chemical_name_mineral')
+    c += cif_value('_chemical_name_common')
+    c += cif_value('_pd_phase_name')
+    c += cif_value('_chemical_formula_sum')
+    c += cif_value('_chemical_formula_weight')
+    c += cif_value('_cell_formula_units_Z')
+
+    # Cell info
+    c += '\n# Cell info\n'
+    c += cif_value('_cell_length_a')
+    c += cif_value('_cell_length_b')
+    c += cif_value('_cell_length_c')
+    c += cif_value('_cell_angle_alpha')
+    c += cif_value('_cell_angle_beta')
+    c += cif_value('_cell_angle_gamma')
+    c += cif_value('_cell_volume')
+
+    # Symmetry info
+    c += '\n# Symmetry info\n'
+    c += cif_value('_symmetry_cell_setting')
+    c += cif_value('_symmetry_space_group_name_H-M')
+    c += cif_value('_symmetry_space_group_name_Hall')
+    c += cif_value('_symmetry_Int_Tables_number')
+    c += '\n'
+    c += cif_loop(['_space_group_symop_operation_xyz', '_symmetry_equiv_pos_as_xyz'])
+
+    # Atom info
+    c += '\n# Atom info\n'
+    c += cif_loop([
+        '_atom_site_label',
+        '_atom_site_type_symbol',
+        #'_atom_site_symmetry_multiplicity',
+        #'_atom_site_Wyckoff_symbol',
+        '_atom_site_fract_x',
+        '_atom_site_fract_y',
+        '_atom_site_fract_z',
+        '_atom_site_U_iso_or_equiv',
+        '_atom_site_occupancy',
+    ])
+
+    if filename is None:
+        return c
+
+    filename, extension = os.path.splitext(filename)
+    filename = filename + '.cif'
+    with open(filename, 'wt') as f:
+        f.write(c)
+    print('CIF written to: %s' % filename)
+
+
+def write_mcif(cifvals, filename=None, comments=None):
+    """
+    Write magnetic .mcif file with values from a cif dict,
+    Only basic items are saved, rather than returning the original file
+    :param cifvals: dict from readcif
+    :param filename: filename to write (use None to return string)
+    :param comments: str comments to write to the file top matter
+    :return: None
+    """
+
+    keys = cifvals.keys()
+
+    def cif_value(name):
+        if name in keys:
+            return '%-40s %-12s\n' % (name, cifvals[name])
+        else:
+            print('%s not in cif dict' % name)
+            return '%-40s %-12s\n' % (name, '?')
+
+    def cif_loop(names):
+        cnames = [name for name in names if name in keys]
+        if len(cnames) < len(names):
+            inames = [name for name in names if name not in keys]
+            print('Loop Items not in cif dict: %s' % inames)
+        if len(cnames) == 0:
+            return ''
+        out = 'loop_\n'
+        out += ''.join(['%s\n' % name for name in cnames])
+        vals = [cifvals[name] for name in cnames]
+        for val_line in zip(*vals):
+            out += ' '.join(['%-12s' % val for val in val_line])
+            out += '\n'
+        return out
+
+    # Top Matter
+    c = '#----------------------------------------------------------------------\n'
+    c += '#   Crystal Structure: %s\n' % (cifvals['FileTitle'] if 'FileTitle' in keys else '')
+    c += '#----------------------------------------------------------------------\n'
+    c += '# CIF created in Dans_Diffraction\n'
+    c += '# Original cif:\n# %s\n' % (cifvals['Filename'] if 'Filename' in keys else 'None')
+
+    # Comments
+    c += '# Comments:\n'
+    if comments:
+        comments = comments.split('\n')
+        c += ''.join(['# %s\n' % comment for comment in comments])
+
+    # Crystal Data
+    c += '\ndata_WRITECIF\n'
+    c += cif_value('_chemical_name_mineral')
+    c += cif_value('_chemical_name_common')
+    c += cif_value('_pd_phase_name')
+    c += cif_value('_chemical_formula_sum')
+    c += cif_value('_chemical_formula_weight')
+    c += cif_value('_cell_formula_units_Z')
+
+    # Cell info
+    c += '\n# Cell info\n'
+    c += cif_value('_cell_length_a')
+    c += cif_value('_cell_length_b')
+    c += cif_value('_cell_length_c')
+    c += cif_value('_cell_angle_alpha')
+    c += cif_value('_cell_angle_beta')
+    c += cif_value('_cell_angle_gamma')
+    c += cif_value('_cell_volume')
+
+    # Symmetry info
+    c += '\n# Symmetry info\n'
+    c += cif_value('_space_group_magn.number_BNS')
+    c += cif_value('_space_group_magn.name_BNS')
+    c += '\n'
+    c += cif_loop([
+        '_space_group_symop_magn_operation.id',
+        '_space_group_symop_magn_operation.xyz',
+        '_space_group_symop_magn_operation.mxmymz',
+    ])
+    c += '\n'
+    c += cif_loop([
+        '_space_group_symop_magn_centering.id',
+        '_space_group_symop_magn_centering.xyz',
+        '_space_group_symop_magn_centering.mxmymz',
+    ])
+
+    # Atom info
+    c += '\n# Atom info\n'
+    c += cif_loop([
+        '_atom_site_label',
+        '_atom_site_type_symbol',
+        #'_atom_site_symmetry_multiplicity',
+        #'_atom_site_Wyckoff_symbol',
+        '_atom_site_fract_x',
+        '_atom_site_fract_y',
+        '_atom_site_fract_z',
+        '_atom_site_U_iso_or_equiv',
+        '_atom_site_occupancy',
+    ])
+
+    # Moment info
+    c += '\n# Atom info\n'
+    c += cif_loop([
+        '_atom_site_moment.label',
+        '_atom_site_moment.crystalaxis_x',
+        '_atom_site_moment.crystalaxis_y',
+        '_atom_site_moment.crystalaxis_z',
+        #'_atom_site_moment.symmform',
+    ])
+
+    if filename is None:
+        return c
+
+    filename, extension = os.path.splitext(filename)
+    filename = filename + '.mcif'
+    with open(filename, 'wt') as f:
+        f.write(c)
+    print('MCIF written to: %s' % filename)
+
+
+'--------------Functions to Read Database files----------------------'
+
+
 def read_atom_properties_file(filedir):
     """
     Reads the text file "Dans Element Properties.txt"
@@ -462,6 +762,8 @@ def read_atom_properties_file(filedir):
 def atom_properties(elements=None, fields=None):
     """
     Loads the atomic properties of a particular atom from a database
+    Atomic properties, scattering lengths, form factors and absorption edges for elements upto and including Uranium.
+    Values are taken from various online sources, see data/README.md for more details.
 
     Usage:
             A = atom_properties() >> returns structured array of all properties for all atoms A[0]['Element']='H'
@@ -474,10 +776,13 @@ def atom_properties(elements=None, fields=None):
     Available properties:
         A = atom_properties()
         print(A.dtype.names)
+        print(a[25]['Element'])
+        >> 'Fe'
 
     Available information includes:
           Z             = Element number
-          Element Name  = Element name
+          Element       = Element symbol
+          Name          = Element name
           Group         = Element Group on periodic table
           Period        = Element period on periodic table
           Block         = Element electronic block (s,p,d,f)
@@ -512,25 +817,26 @@ def atom_properties(elements=None, fields=None):
           M1...         = x-ray absorption edge
     """
 
-    atomfile = os.path.join(datadir, 'Dans Element Properties.txt')
     try:
-        data = np.genfromtxt(atomfile, skip_header=6, dtype=None, names=True, encoding='ascii')
+        data = np.genfromtxt(ATOMFILE, skip_header=6, dtype=None, names=True, encoding='ascii')
     except TypeError:
         # Numpy version < 1.14
-        data = np.genfromtxt(atomfile, skip_header=6, dtype=None, names=True)
+        data = np.genfromtxt(ATOMFILE, skip_header=6, dtype=None, names=True)
 
     if elements is not None:
         # elements must be a list e.g. ['Co','O']
         elements = np.char.lower(np.asarray(elements).reshape(-1))
-        indx = [None] * len(elements)
-        for n in range(len(data)):
-            d = data[n]
-            fileelement = d['Element'].lower()
-            if fileelement in elements:
-                for m in range(len(elements)):
-                    if fileelement == elements[m]:
-                        indx[m] = n
-        data = data[indx]
+        all_elements = [el.lower() for el in data['Element']]
+        # This will error if the required element doesn't exist
+        try:
+            index = [all_elements.index(el) for el in elements]
+        except ValueError as ve:
+            raise Exception('Element not available: %s' % ve)
+        data = data[index]
+        if np.any(data['Z'] > 92):
+            msg = 'Element %s does not have complete atomic properties, scattering calculations will be inaccurate.'
+            for el in data[data['Z'] > 92]:
+                warn(msg % el['Element'])
 
     if fields is None:
         return data
@@ -557,11 +863,27 @@ def print_atom_properties(elements=None):
     return out
 
 
+def neutron_scattering_length(element):
+    """
+    Return neutron scattering length, b, in fm
+    Uses bound coherent scattering length from NIST
+    https://www.ncnr.nist.gov/resources/n-lengths/
+     b = neutron_scattering_length('Co')
+    :param element: [n*str] list or array of elements
+    :return: [n] array of scattering lengths
+    """
+    b = atom_properties(element, ['Coh_b'])
+    return b
+
+
 def xray_scattering_factor(element, Qmag=0):
     """
     Read X-ray scattering factor table, calculate f(|Q|)
     Uses the oefficients for analytical approximation to the scattering factors - ITC, p578
      Qff = read_xsf(element,Qmag=[0])
+    :param element: [n*str] list or array of elements
+    :param Qmag: [m] array of wavevector distance, in A^-1
+    :return: [m*n] array of scattering factors
     """
 
     # Qmag should be a 1D array
@@ -631,30 +953,6 @@ def magnetic_form_factor(element, Qmag=0.):
     return Qff
 
 
-def debyewaller(uiso, Qmag=[0]):
-    """
-    Calculate the debye waller factor for a particular Q
-     T = debyewaller(uiso,Qmag=[0])
-
-        T = exp( -2*pi^2*Uiso/d^2 )
-        T = exp( -Uiso/2Q^2 )
-    """
-
-    # Qmag should be a 1D array
-    Qmag = np.asarray(Qmag).reshape(-1)
-
-    uiso = np.asarray(uiso, dtype=np.float).reshape(1, -1)
-    Qmag = np.asarray(Qmag, dtype=np.float).reshape(-1, 1)
-    Tall = np.exp(-0.5 * np.dot(Qmag, uiso))
-    # Tall = np.zeros([len(Qmag),len(uiso)])
-    # Generate the approximation for each atom
-    # for n in range(len(uiso)):
-    #    for m in range(len(Qmag)):
-    #        # Definition of Debye-Waller factor from Vesta user manual p45
-    #        Tall[m,n] = dnp.exp(-0.5*uiso[n]*Qmag[m])
-    return Tall
-
-
 def attenuation(element_z, energy_keV):
     """
      Returns the x-ray mass attenuation, u/p, in cm^2/g
@@ -671,22 +969,215 @@ def attenuation(element_z, energy_keV):
     energies = xma_data[:, 0] / 1000.
     out = np.zeros([len(energy_keV), len(element_z)])
     for n, z in enumerate(element_z):
+        # Interpolating the log values is much more reliable
+        out[:, n] = np.exp(np.interp(np.log(energy_keV), np.log(energies), np.log(xma_data[:, z])))
         out[:, n] = np.interp(energy_keV, energies, xma_data[:, z])
     if len(element_z) == 1:
         return out[:, 0]
     return out
 
 
+def atomic_scattering_factor(element, energy_kev=None):
+    """
+    Read atomic scattering factor table, giving f1+f2 for different energies
+    From: http://henke.lbl.gov/optical_constants/asf.html
+    :param element: str name of element
+    :param energy_kev: float or list energy in keV (None to return original, uninterpolated list)
+    :return: f1, f2
+    """
+    asf_file = os.path.join(datadir, 'atomic_scattering_factors.npy')
+    asf = np.load(asf_file, allow_pickle=True)
+    asf = asf.item()
+    energy = np.array(asf[element]['energy'])/1000. # eV -> keV
+    f1 = np.array(asf[element]['f1'])
+    f2 = np.array(asf[element]['f2'])
+    f1[f1 < -1000] = np.nan
+    f2[f2 < -1000] = np.nan
+
+    if energy_kev is None:
+        return energy, f1, f2
+
+    # Interpolate values
+    if1 = np.interp(energy_kev, energy, f1)
+    if2 = np.interp(energy_kev, energy, f2)
+    return if1, if2
+
+
+def spacegroups():
+    """
+    Return a dict of all space groups
+    Loaded from json file created from the Bilbao crystallographic server: https://www.cryst.ehu.es/
+    :return: dict with keys:
+        'space group number'
+        'space group name html'
+        'space group name'
+        'web address generator'
+        'web address wyckoff'
+        'web address site check'     # address % (u, v, w),
+        'general positions'
+        'magnetic space groups',
+        'positions centring'
+        'positions multiplicity'
+        'positions wyckoff letter'
+        'positions symmetry'
+        'positions coordinates'
+        'subgroup number',
+        'subgroup name',
+        'subgroup index',
+        'subgroup type'
+    """
+    sg_file = os.path.join(datadir, 'SpaceGroups.json')
+    with open(sg_file, 'r') as fp:
+        sg_dict = json.load(fp)
+    return sg_dict
+
+
+def spacegroup(sg_number):
+    """
+    Return a dict of information for a space group
+    :param sg_number: int space group number, as in the international tables of Crystallography
+    :return: dict
+    """
+    sg_number = str(int(sg_number))
+    sg_dict = spacegroups()
+    return sg_dict[sg_number]
+
+
+def spacegroup_list(sg_numbers=None):
+    """
+    Return a structured list of space groups with general operations
+    :param sg_numbers: list of space group numbers (None for whole list)
+    :return: str
+    """
+    if sg_numbers is None:
+        sg_numbers = range(1,231)
+    sg_numbers = np.asarray(sg_numbers, dtype=str).reshape(-1)
+
+    sg_dict = spacegroups()
+
+    out = ''
+    fmt = '%3d %-10s : %3d : %s\n'
+    for sgn in sg_numbers:
+        sg = sg_dict[sgn]
+        num = sg['space group number']
+        name = sg['space group name']
+        sym = ', '.join(sg['general positions'])
+        nsym = len(sg['general positions'])
+        out += fmt % (num, name, nsym, sym)
+    return out
+
+
+def spacegroup_subgroups(sg_number):
+    """
+    Return dict of maximal subgroups for spacegroup
+    :param sg_number: space group number (1-230)
+    :return: dict
+    """
+    all_sg = spacegroups()
+    sg_number = str(int(sg_number))
+    sg_dict = all_sg[sg_number]
+    subgroups = sg_dict["subgroup number"]
+    return [all_sg[num] for num in subgroups]
+
+
+def spacegroup_subgroups_list(sg_number):
+    """
+    Return str of maximal subgroups for spacegroup
+    :param sg_number: space group number (1-230)
+    :return: str
+    """
+    sg_dict = spacegroup(sg_number)
+    sub_num = sg_dict["subgroup number"]
+    sub_name = sg_dict["subgroup name"]
+    sub_index = sg_dict["subgroup index"]
+    sub_type = sg_dict["subgroup type"]
+    out = ''
+    fmt = 'Parent: %3s Subgroup: %3s  %-10s  Index: %3s  Type: %2s\n'
+    for num, name, index, stype in zip(sub_num, sub_name, sub_index, sub_type):
+        out += fmt % (sg_number, num, name, index, stype)
+    return out
+
+
+def spacegroups_magnetic(sg_number):
+    """
+    Returns dict of magnetic space groups for required space group
+    :param sg_number: space group number (1-230)
+    :return: dict
+    """
+    sg_dict = spacegroup(sg_number)
+    msg_numbers = sg_dict['magnetic space groups']
+
+    msg_file = os.path.join(datadir, 'SpaceGroupsMagnetic.json')
+    with open(msg_file, 'r') as fp:
+        msg_dict = json.load(fp)
+    return [msg_dict[num] for num in msg_numbers]
+
+
+def spacegroup_magnetic(msg_number):
+    """
+    Return dict of magnetic spacegroup, given magnetic spacegroup number
+    :param msg_number: magnetic space group number e.g. 61.433
+    :return: dict with keys:
+        'parent number': sg,
+        'space group number': number,
+        'space group name': label,
+        'setting': setting,
+        'type name': type_name,
+        'related group': rel_number,
+        'related name': rel_name,
+        'related setting': rel_setting,
+        'operators general': xyz_op,
+        'operators magnetic': mxmymz_op,
+        'operators time': time,
+    """
+    msg_number = np.asarray(msg_number, dtype=str).reshape(-1)
+
+    msg_file = os.path.join(datadir, 'SpaceGroupsMagnetic.json')
+    with open(msg_file, 'r') as fp:
+        msg_dict = json.load(fp)
+
+    if None in msg_number:
+        return msg_dict
+    elif len(msg_number) == 1:
+        return msg_dict[msg_number[0]]
+    else:
+        return [msg_dict[num] for num in msg_number]
+
+
+def spacegroup_magnetic_list(sg_number):
+    """
+    Return str list of magnetic space groups
+    :param sg_number: space group number (1-230)
+    :return: str
+    """
+    mag_spacegroups = spacegroups_magnetic(sg_number)
+    out = ''
+    fmt = 'Parent: %3s Magnetic: %-10s  %-10s  Setting: %3s %30s  Operators: %s\n'
+    for sg in mag_spacegroups:
+        parent = sg['parent number']
+        number = sg['space group number']
+        name = sg['space group name']
+        setting = sg['setting']
+        typename = sg['type name']
+        #ops = sg['operators magnetic']
+        ops = sg['positions magnetic']
+        out += fmt % (parent, number, name, setting, typename, ops)
+    return out
+
+
 '--------------Element Properties & Charge----------------------'
 
 
-def element_symbol(element_Z):
+def element_symbol(element_Z=None):
     """
     Returns the element sympol for element_Z
-    :param element_Z: int
+    :param element_z: int or array or None for all elements
     :return: str
     """
     symbols = atom_properties(None, 'Element')
+    if element_Z is None:
+        return symbols
+    element_Z = np.asarray(element_Z).reshape(-1)
     return symbols[element_Z-1]
 
 
@@ -702,41 +1193,120 @@ def element_z(element):
     return z
 
 
-def orbital_configuration(element_Z):
+def element_name(element=None):
     """
-    Returns the orbital configuration of an element as a list of strings
-    :param element_Z: int
-    :return: ['1s2', '2s2', '2p6', ...]
+    Returns the element name
+    :param element: str
+    :return: int
     """
-
-    config = atom_properties(element_symbol(element_Z), 'Config')
-    return config[0].split('.')
+    name = atom_properties(element, 'Name')
+    if len(name) == 1:
+        return name[0]
+    return name
 
 
 def split_element_symbol(element):
     """
     From element symbol, split charge and occupancy
       symbol, occupancy, charge = split_element_symbol('Co3+')
+    Any numbers appended by +/- are taken as charge, otherwise they are counted as occupancy.
+    e.g.    element     >   symbol  |   occupancy   |   charge
+            Co3+            Co          1.              3
+            3.2O2-          O           3.2             -2
+            fe3             Fe          3               0
     :param element: str
     :return: symbol: str
     :return: occupancy: float
     :return: charge: float
     """
-
-    atom_n = re.findall('[A-Z][a-z]?|[0-9]+[.]?[0-9]*', element)
-    symbol = atom_n[0]
-    charge = 0
-    occupancy = 1
-    if len(atom_n) > 1:
-        if '-' in element:
-            charge = -float(atom_n[1])
-        elif '+' in element:
-            charge = float(atom_n[1])
+    # Find element
+    symbol = element_regex.findall(element)[0]
+    # Find charge
+    find_charge = re.findall('[\d\.]+[\+-]', element)
+    if len(find_charge) > 0:
+        chargestr = find_charge[0]
+        element = element.replace(chargestr, '')
+        if '-' in chargestr:
+            charge = -float(chargestr[:-1])
         else:
-            occupancy = float(atom_n[1])
+            charge = float(chargestr[:-1])
+    else:
+        charge = 0.
+    # Find occupancy
+    find_occ = re.findall('[\d\.]+', element)
+    if len(find_occ) > 0:
+        occupancy = float(find_occ[0])
+    else:
+        occupancy = 1.
     return symbol, occupancy, charge
 
 
+def element_charge_string(symbol, occupancy=1.0, charge=0.0, latex=False):
+    """
+    Return formatted string of element with occupancy and charge
+    :param symbol: str - element string
+    :param occupancy: float - element occupancy or number of elements
+    :param charge: float - element charge
+    :param latex: if True, returns string formatted with latex tags
+    :return: str
+    """
+    if abs(charge) < 0.01:
+        chstr = ''
+    elif abs(charge - 1) < 0.01:
+        chstr = '+'
+    elif abs(charge + 1) < 0.01:
+        chstr = '-'
+    elif charge > 0:
+        chstr = '%0.3g+' % charge
+    else:
+        chstr = '%0.3g-' % abs(charge)
+
+    if latex:
+        chstr = '$^{%s}$' % chstr
+
+    if np.abs(occupancy - 1) < 0.01:
+        outstr = '%s%s' % (symbol, chstr)
+    else:
+        outstr = '%0.3g[%s%s]' % (occupancy, symbol, chstr)
+    return outstr
+
+
+def split_compound(compound_name):
+    """
+    Convert a molecular or compound name into a list of elements and numbers.
+    Assumes all element multiplications are to the LEFT of the element name.
+    Values in brackets are multiplied out.
+    E.g.
+        split_compound('Mn0.3(Fe3.6(Co1.2)2)4(Mo0.7Pr44)3')
+        >> ['Mn0.3', 'Fe14.4', 'Co9.6', 'Mo2.1', 'Pr132']
+    :param compound_name: str
+    :return: list of str
+    """
+    regex_element_num = re.compile('|'.join(['%s[\d\.]*' % el for el in ELEMENT_LIST]))
+    # Multiply out brackets
+    compound_name = fg.replace_bracket_multiple(compound_name)
+    return regex_element_num.findall(compound_name)
+
+
+def orbital_configuration(element, charge=None):
+    """
+    Returns the orbital configuration of an element as a list of strings
+    :param element: str, element name, e.g. 'Fe' or 'Fe3+' or '0.8Fe2+'
+    :param charge: int, element charge (overwrites charge given in element)
+    :return: ['1s2', '2s2', '2p6', ...]
+    """
+    symbol, occupancy, charge_str = split_element_symbol(element)
+    z = element_z(symbol)
+    if charge is None:
+        charge = charge_str
+    newz = z - int(charge) # floor
+    if newz < 1: newz = 1
+    if newz > 118: newz = 118
+    element = element_symbol(newz)
+    config = atom_properties(element, 'Config')
+    return config[0].split('.')
+
+
 def default_atom_charge(element):
     """
     Returns the default charge value for the element
@@ -866,22 +1436,7 @@ def count_charges(list_of_elements, occupancy=None, divideby=1, latex=False):
 
     outstr = []
     for a in ats:
-        if np.abs(chno[a]-1) < 0.01:
-            chstr = '+'
-        elif np.abs(chno[a]+1) < 0.01:
-            chstr = '-'
-        elif chno[a] > 0:
-            chstr = '%0.2g+' % chno[a]
-        else:
-            chstr = '%0.2g-' % abs(chno[a])
-
-        if latex:
-            chstr = '$^{%s}$' % chstr
-
-        if np.abs(atno[a]-1) < 0.01:
-            outstr += ['%s%s' % (a, chstr)]
-        else:
-            outstr += ['%0.2g[%s%s]' % (atno[a], a, chstr)]
+        outstr += [element_charge_string(a, atno[a], chno[a], latex)]
     return outstr
 
 
@@ -1400,6 +1955,17 @@ def dspace2q(dspace):
     return 2 * np.pi / dspace
 
 
+def resolution2energy(res, twotheta=180.):
+    """
+    Calcualte the energy required to achieve a specific resolution at a given two-theta
+    :param res: measurement resolution in A (==d-spacing)
+    :param twotheta: Bragg angle in Degrees
+    :return: float
+    """
+    theta = twotheta * np.pi / 360  # theta in radians
+    return (fg.h * fg.c * 1e10) / (res * np.sin(theta) * fg.e * 2 * 1000.)
+
+
 def wave2energy(wavelength):
     """
     Converts wavelength in A to energy in keV
@@ -1483,9 +2049,16 @@ def symmetry_ops2magnetic(operations):
     # convert string to list
     if type(operations) is str:
         operations = [operations]
-
-    rem = ['+1/2', '+1/3', '+2/3', '+1/6', '+5/6']
-    return [fg.multi_replace(sp, rem, '') for sp in operations]
+    # Use RegEx to find translations
+    mag_op = []
+    for op in operations:
+        translations = re.findall('[\+\-]?\d/\d[\+\-]?', op)
+        op = fg.multi_replace(op, translations, '')
+        # also remove +/-1
+        translations = re.findall('[\+\-]?\d+?[\+\-]?', op)
+        op = fg.multi_replace(op, translations, '')
+        mag_op += [op]
+    return mag_op
 
 
 def hkl2str(hkl):
@@ -1517,6 +2090,30 @@ def cut2powder(qx, qy, qz, cut):
 '--------------------------Misc Crystal Programs------------------------'
 
 
+def str2element(string):
+    """Finds element name in string"""
+    result = element_regex.findall(string)
+    if len(result) == 0:
+        return False
+    return result[0].capitalize()
+
+
+def debyewaller(uiso, Qmag=0):
+    """
+    Calculate the debye waller factor for a particular Q
+     T = debyewaller(uiso,Qmag=[0])
+
+        T = exp( -2*pi^2*Uiso/d^2 )
+        T = exp( -Uiso/2Q^2 )
+    """
+
+    uiso = np.asarray(uiso, dtype=np.float).reshape(1, -1)
+    Qmag = np.asarray(Qmag, dtype=np.float).reshape(-1, 1)
+
+    Tall = np.exp(-0.5 * np.dot(Qmag, uiso))
+    return Tall
+
+
 def powder_average(tth, energy_kev):
     """
     Return the powder average correction for intensities at a given two-theta
