@@ -15,8 +15,8 @@ Usage:
     - from Dans_Diffraction import functions_general as fg
 
 
-Version 1.5
-Last updated: 27/03/20
+Version 1.7
+Last updated: 12/05/20
 
 Version History:
 06/01/18 1.0    Program created from DansGeneralProgs.py V2.3
@@ -25,6 +25,8 @@ Version History:
 31/10/18 1.3    Added complex2str
 20/08/19 1.4    Added search_dict_lists
 27/03/20 1.5    Corrected error in gauss for 1d case when centre /= 0
+05/05/20 1.6    New version of readstfm, allows E powers and handles non-numbers.
+12/05/20 1.7    Added sph2cart, replace_bracket_multiple
 
 @author: DGPorter
 """
@@ -32,7 +34,7 @@ Version History:
 import sys, os, re
 import numpy as np
 
-__version__ = '1.5'
+__version__ = '1.7'
 
 # File directory
 directory = os.path.abspath(os.path.dirname(__file__))
@@ -152,6 +154,29 @@ def ang(a, b, deg=False):
     return angle
 
 
+def cart2sph(xyz, deg=False):
+    """
+    Convert coordinates in cartesian to coordinates in spherical
+    https://en.wikipedia.org/wiki/Spherical_coordinate_system
+    ISO convention used.
+        theta = angle from Z-axis to X-axis
+          phi = angle from X-axis to component in XY plane
+    :param xyz: [n*3] array of [x,y,z] coordinates
+    :param deg: if True, returns angles in degrees
+    :return: [r, theta, phi]
+    """
+    xyz = np.asarray(xyz).reshape(-1, 3)
+    xy = xyz[:, 0] ** 2 + xyz[:, 1] ** 2
+    r = mag(xyz)
+    theta = np.arctan2(np.sqrt(xy), xyz[:, 2])  # for elevation angle defined from Z-axis down
+    #theta = np.arctan2(xyz[:,2], np.sqrt(xy))  # for elevation angle defined from XY-plane up
+    phi = np.arctan2(xyz[:, 1], xyz[:, 0])
+    if deg:
+        theta = np.rad2deg(theta)
+        phi = np.rad2deg(phi)
+    return np.vstack((r, theta, phi)).T
+
+
 def rot3D(A, alpha=0., beta=0., gamma=0.):
     """Rotate 3D vector A by euler angles
         A = rot3D(A,alpha=0.,beta=0.,gamma=0.)
@@ -219,6 +244,30 @@ def rotmat(a, b):
     return U
 
 
+def group(A, tolerance=0.0001):
+    """
+    Group similear values in an array, returning the group and indexes
+    array will be sorted so lowest numbers are grouped first
+      group_index, group_values, group_counts = group([2.1, 3.0, 3.1, 1.00], 0.1)
+    group_values = [1., 2., 3.] array of grouped numbers (rounded)
+    array_index = [1, 2, 2, 0] array matching A values to groups, such that A ~ group_values[group_index]
+    group_index = [3, 0, 1] array matching group values to A, such that group_values ~ A[group_index]
+    group_counts = [1, 1, 2] number of iterations of each item in group_values
+    :param A: list or numpy array of numbers
+    :param tolerance: values within this number will be grouped
+    :return: group_values, array_index, group_index, group_counts
+    """
+    A = np.asarray(A, dtype=np.float).reshape(-1)
+    idx = np.argsort(A)
+    rtn_idx = np.argsort(idx)
+    A2 = np.round(A/tolerance)*tolerance
+    groups, indices, inverse, counts = np.unique(A2[idx], return_index=True, return_inverse=True, return_counts=True)
+    # groups = A[idx][indices]  # return original, not rounded values
+    array_index = inverse[rtn_idx]
+    group_index = idx[indices]
+    return groups, array_index, group_index,  counts
+
+
 def unique_vector(vecarray, tol=0.05):
     """
     Find unique vectors in an array within a certain tolerance
@@ -262,6 +311,25 @@ def unique_vector(vecarray, tol=0.05):
     return newarray, uniqueidx, matchidx
 
 
+def distance2line(line_start, line_end, point):
+    """
+    Calculate distance from a line between the start and end to an arbitary point in space
+    :param line_start: array, position of the start of the line
+    :param line_end:  array, position of the end of the line
+    :param point: array, arbitary position in space
+    :return: float
+    """
+    line_start = np.asarray(line_start)
+    line_end = np.asarray(line_end)
+    point = np.asarray(point)
+
+    line_diff = line_end - line_start
+    unit_line = line_diff / np.sqrt(np.sum(line_diff ** 2))
+
+    vec_arb = (line_start - point) - np.dot((line_start - point), unit_line) * unit_line
+    return np.sqrt(np.sum(vec_arb ** 2))
+
+
 def find_index(A, value):
     """
     Return the index of the closest number in array A to value
@@ -524,27 +592,40 @@ def readstfm(string):
     """
     Read numbers written in standard form: 0.01(2), return value and error
     Read numbers from string with form 0.01(2), returns floats 0.01 and 0.02
+    Errors and values will return 0 if not given.
 
     E.G.
     readstfm('0.01(2)') = (0.01, 0.02)
     readstfm('1000(300)') = (1000.,300.)
+    readstfm('1.23(3)E4') = (12300.0, 300.0)
     """
 
-    values = re.findall('[-0-9.]+', string)
+    values = re.findall('[-0-9.]+|\([-0-9.]+\)', string)
+    if len(values) > 0 and '(' not in values[0] and values[0] != '.':
+        value = values[0]
+    else:
+        value = '0'
 
-    if values[0] == '.':
-        values[0] = '0'
-    value = float(values[0])
-    error = 0.
+    # Determine number of decimal places for error
+    idx = value.find('.') # returns -1 if . not found
+    if idx > -1:
+        pp = idx - len(value) + 1
+    else:
+        pp = 0
+    value = float(value)
 
-    if len(values) > 1:
-        error = float(values[1])
+    error = re.findall('\([-0-9.]+\)', string)
+    if len(error) > 0:
+        error = abs(float(error[0].strip('()')))
+        error = error * 10 ** pp
+    else:
+        error = 0.
 
-        # Determine decimal place
-        idx = values[0].find('.')  # returns -1 if no decimal
-        if idx > -1:
-            pp = idx - len(values[0]) + 1
-            error = error * 10 ** pp
+    power = re.findall('(?:[eE]|x10\^|\*10\^|\*10\*\*)([+-]?\d*\.?\d+)', string)
+    if len(power) > 0:
+        power = float(power[0])
+        value = value * 10 ** power
+        error = error * 10 ** power
     return value, error
 
 
@@ -658,6 +739,61 @@ def multi_replace(string, old=[], new=[]):
     return string
 
 
+def replace_bracket_multiple(name):
+    """
+    Replace any numbers in brackets with numbers multipled by bracket multiplyer
+    Assumes bracket multiplier is on the left side
+    e.g.
+        replace_bracket_multiple('Mn0.3(Fe3.6(Co1.2)2)4(Mo0.7Pr44)3')
+        >> 'Mn0.3Fe14.4Co9.6Mo2.1Pr132'
+    :param name: str
+    :return: str
+    """
+    """
+    To do:
+     - Multiply by fraction (regex for '/number')
+     - Multiple on right hand side
+    """
+    # Regex:
+    regex_num = re.compile('[\d\.]+')
+    regex_bracket_n = re.compile('\)[\d\.]+')
+
+    # Find outside brackets
+    bracket = []
+    start_idx = []
+    level=0
+    for n, s in enumerate(name):
+        if s in ['(', '[', '{']:
+            start_idx += [n]
+            level += 1
+        elif s in [')', ']', '}']:
+            level -= 1
+            if level == 0:
+                num = regex_bracket_n.findall(name[n:])
+                if len(num) > 0:
+                    bracket_end = n + len(num[0])
+                    num = float(num[0][1:])
+                else:
+                    bracket_end = n+1
+                    num = 1.0
+                bracket += [(
+                    name[start_idx[0]+1:n],  # insde brackets
+                    name[start_idx[0]:bracket_end],  # str to replace
+                    num  # multiplication appending bracket
+                )]
+                start_idx = []
+
+    for numstr, repstr, num in bracket:
+        # Run recursivley to get inner brackets
+        numstr = replace_bracket_multiple(numstr)
+        # Replace each number by it's multiple
+        for oldnum in regex_num.findall(numstr):
+            numstr = numstr.replace(oldnum, '%0.3g'%(float(oldnum)*num))
+        # Replace in original string
+        name = name.replace(repstr, numstr)
+    return name
+
+
 def nice_print(precision=4, linewidth=300):
     """
     Sets default printing of arrays to a nicer format
