@@ -12,7 +12,7 @@ if sys.version_info[0] < 3:
 else:
     import tkinter as tk
     from tkinter import filedialog
-    #from tkinter import messagebox
+    from tkinter import messagebox
 
 # Internal functions
 from ..classes_crystal import Crystal
@@ -21,8 +21,8 @@ from ..classes_fdmnes import fdmnes_checker
 from .. import functions_general as fg
 from .. import functions_plotting as fp
 from .. import functions_crystallography as fc
-from .basic_widgets import StringViewer
-from .basic_widgets import (TF, BF, SF, LF, HF,
+from .basic_widgets import StringViewer, SelectionBox
+from .basic_widgets import (TF, BF, SF, LF, HF, TTF,
                             bkg, ety, btn, opt, btn2,
                             btn_active, opt_active, txtcol,
                             btn_txt, ety_txt, opt_txt)
@@ -64,13 +64,13 @@ class CrystalGui:
         # Filename (variable)
         f_file = tk.Frame(frame)
         f_file.pack(side=tk.TOP, expand=tk.YES, fill=tk.X)
-        self.file = tk.StringVar(frame, self.xtl.filename)
+        self.file = tk.StringVar(frame, self.short_file())
         var = tk.Label(f_file, text='CIF file:', font=SF, width=10)
         var.pack(side=tk.LEFT, expand=tk.NO)
-        var = tk.Label(f_file, textvariable=self.file, font=TF)
-        var.pack(side=tk.LEFT, expand=tk.YES)
+        var = tk.Label(f_file, textvariable=self.file, width=40, font=TF)
+        var.pack(side=tk.LEFT, expand=tk.NO, padx=3)
         var = tk.Button(f_file, text='Load CIF', font=BF, bg=btn, activebackground=btn_active, command=self.fun_loadcif)
-        var.pack(side=tk.LEFT, expand=tk.NO, padx=5)
+        var.pack(side=tk.RIGHT, expand=tk.NO, padx=5)
 
         # Name (variable)
         f_name = tk.Frame(frame)
@@ -127,6 +127,9 @@ class CrystalGui:
         var = tk.Button(f_but, text='Super\nStructure', bg=btn, activebackground=btn_active, font=BF,
                         command=self.fun_superstructure)
         var.pack(side=tk.LEFT)
+        var = tk.Button(f_but, text='Write\nCIF', bg=btn, activebackground=btn_active, font=BF,
+                        command=self.fun_writecif)
+        var.pack(side=tk.LEFT)
 
         # Buttons 3
         f_but = tk.Frame(frame)
@@ -137,9 +140,10 @@ class CrystalGui:
         var = tk.Button(f_but, text='Multiple\nScattering', bg=btn, activebackground=btn_active, font=BF,
                         command=self.fun_multiple_scattering)
         var.pack(side=tk.LEFT)
-        var = tk.Button(f_but, text='Tensor\nScattering', bg=btn, activebackground=btn_active, font=BF,
-                        command=self.fun_tensor_scattering)
-        var.pack(side=tk.LEFT)
+        # Remove Tensor-Scattering 26/05/20
+        #var = tk.Button(f_but, text='Tensor\nScattering', bg=btn, activebackground=btn_active, font=BF,
+        #                command=self.fun_tensor_scattering)
+        #var.pack(side=tk.LEFT)
         if fdmnes_checker():
             var = tk.Button(f_but, text='Run\nFDMNES', bg=btn, activebackground=btn_active, font=BF,
                             command=self.fun_fdmnes)
@@ -156,22 +160,60 @@ class CrystalGui:
     ############################## FUNCTIONS ##########################################
     ###################################################################################
     def fun_set(self):
-        self.file.set(self.xtl.filename)
+        self.file.set(self.short_file())
         self.name.set(self.xtl.name)
 
     def fun_get(self):
         self.xtl.name = self.name.get()
 
+    def short_file(self):
+        path, name = os.path.split(self.xtl.filename)
+        path, short_path = os.path.split(path)
+        return '/'.join([short_path, name])
+
     def fun_loadcif(self):
         # root = Tk().withdraw() # from Tkinter
         defdir = os.path.join(os.path.dirname(__file__), 'Structures')
         # defdir = os.path.expanduser('~')
-        filename = filedialog.askopenfilename(initialdir=defdir,
-                                              filetypes=[('cif file', '.cif'), ('magnetic cif', '.mcif'),
-                                                         ('All files', '.*')])  # from tkFileDialog
+        filename = filedialog.askopenfilename(
+            title='Select CIF to open',
+            initialdir=defdir,
+            filetypes=[('cif file', '.cif'), ('magnetic cif', '.mcif'), ('All files', '.*')],
+            parent=self.root
+        )
         if filename:
-            self.xtl = Crystal(filename)
-            self.fun_set()
+            # Check cif
+            cifvals = fc.readcif(filename)
+            if fc.cif_check(cifvals):
+                self.xtl = Crystal(filename)
+                self.fun_set()
+            else:
+                messagebox.showinfo(
+                    title='Dans_Diffraction',
+                    message='This cif file is missing a key parameter',
+                    parent=self.root
+                )
+                self.xtl = Crystal(filename)
+                self.fun_set()
+
+    def fun_writecif(self, inifile=None):
+        if inifile is None:
+            inifile = '%s.cif' % self.xtl.name
+        defdir = os.path.join(os.path.dirname(__file__), 'Structures')
+        filename = filedialog.asksaveasfilename(
+            title='Save structure as CIF',
+            initialfile=inifile,
+            initialdir=defdir,
+            filetypes=[('cif file', '.cif'), ('magnetic cif', '.mcif'), ('All files', '.*')],
+            parent=self.root
+        )
+        if filename:
+            self.xtl.write_cif(filename)
+            messagebox.showinfo(
+                title='Dans_Diffraction',
+                message='Structure saved as:\n"%s"' % filename,
+                parent=self.root
+            )
 
     def fun_loadstruct(self, event=None):
         """Load from structure_list"""
@@ -179,7 +221,7 @@ class CrystalGui:
             self.xtl = getattr(self.structure_list, self.struct_list.get()).build()
             self.fun_set()
 
-    def update_name(self):
+    def update_name(self, event=None):
         newname = self.name.get()
         self.xtl.name = newname
 
@@ -207,8 +249,8 @@ class CrystalGui:
 
     def fun_info(self):
         """Display Crystal info"""
-        string = self.xtl.info()
-        StringViewer(string, self.xtl.name)
+        string = '%s\n%s' % (self.xtl.filename, self.xtl.info())
+        StringViewer(string, self.xtl.name, width=60)
 
     def fun_plotxtl(self):
         self.fun_set()
@@ -539,6 +581,31 @@ class SymmetryGui:
             activeBackground=opt_active,
             activeForeground=txtcol)
 
+        frm1 = tk.Frame(self.root)
+        frm1.pack(side=tk.TOP)
+
+        # Spacegroup entry
+        var = tk.Label(frm1, text='Spacegroup: ', font=TTF)
+        var.pack(side=tk.LEFT)
+        self.spacegroup = tk.StringVar(frm1, 'P1')
+        self.spacegroup_number = tk.StringVar(frm1, '1')
+        var = tk.Entry(frm1, textvariable=self.spacegroup_number, font=TF, width=4, bg=ety, fg=ety_txt)
+        var.pack(side=tk.LEFT)
+        var.bind('<Return>', self.fun_spacegroup)
+        var.bind('<KP_Enter>', self.fun_spacegroup)
+        var = tk.Label(frm1, textvariable=self.spacegroup, width=14)
+        var.pack(side=tk.LEFT, padx=5)
+        # Spacegroup Buttons
+        var = tk.Button(frm1, text='Choose\nSpacegroup', command=self.fun_ch_spacegroup, height=2, font=BF, bg=btn,
+                        activebackground=btn_active)
+        var.pack(side=tk.LEFT, padx=5, pady=5)
+        var = tk.Button(frm1, text='Choose\nSubgroup', command=self.fun_ch_subgroup, height=2,
+                        font=BF, bg=btn, activebackground=btn_active)
+        var.pack(side=tk.LEFT, padx=5, pady=5)
+        var = tk.Button(frm1, text='Choose Magnetic\nSpacegroup', command=self.fun_ch_maggroup, height=2,
+                        font=BF, bg=btn, activebackground=btn_active)
+        var.pack(side=tk.LEFT, padx=5, pady=5)
+
         frame = tk.Frame(self.root)
         frame.pack(side=tk.TOP)
 
@@ -582,8 +649,6 @@ class SymmetryGui:
         self.text_3.config(yscrollcommand=self.fun_move)
         # scany.config(command=self.text1.yview)
 
-        self.fun_set()
-
         frm1 = tk.Frame(self.root)
         frm1.pack(side=tk.TOP)
 
@@ -616,6 +681,8 @@ class SymmetryGui:
                         command=self.fun_symhkl)
         var.pack(side=tk.LEFT)
 
+        self.fun_set()
+
     def update(self):
         """Update crystal"""
 
@@ -669,6 +736,10 @@ class SymmetryGui:
 
     def fun_set(self):
         """Generate the text box"""
+        # Clear boxes
+        self.text_1.delete('1.0', tk.END)
+        self.text_2.delete('1.0', tk.END)
+        self.text_3.delete('1.0', tk.END)
         # Build string
         Nops = len(self.Symmetry.symmetry_operations)
         symstr = ''
@@ -685,6 +756,47 @@ class SymmetryGui:
         self.text_2.insert(tk.END, symstr)
         self.text_3.insert(tk.END, magstr)
 
+        # Update spacegroup name + number
+        self.spacegroup.set(self.Symmetry.spacegroup)
+        self.spacegroup_number.set(self.Symmetry.spacegroup_number)
+
+    def fun_spacegroup(self, event=None):
+        """Load spacegroup symmetry"""
+        sgn = int(self.spacegroup_number.get())
+        self.Symmetry.load_spacegroup(sgn)
+        self.fun_set()
+
+    def fun_ch_spacegroup(self):
+        """Button Select Spacegroup"""
+        current = int(self.spacegroup_number.get())
+        sg_list = fc.spacegroup_list().split('\n')
+        current_selection = [sg_list[current-1]]
+        selection = SelectionBox(self.root, sg_list, current_selection, 'Select SpaceGroup', False).show()
+        if len(selection) > 0:
+            new_sg = int(selection[0].split()[0])
+            self.Symmetry.load_spacegroup(new_sg)
+            self.fun_set()
+
+    def fun_ch_subgroup(self):
+        """Button select subgroup"""
+        current = int(float(self.spacegroup_number.get()))  # str > float > int
+        sbg_list = fc.spacegroup_subgroups_list(current).split('\n')
+        selection = SelectionBox(self.root, sbg_list, [], 'Select SpaceGroup Subgroup', False).show()
+        if len(selection) > 0:
+            new_sg = int(selection[0].split()[3])
+            self.Symmetry.load_spacegroup(new_sg)
+            self.fun_set()
+
+    def fun_ch_maggroup(self):
+        """Button Select Magnetic Spacegroup"""
+        current = int(self.spacegroup_number.get())
+        msg_list = fc.spacegroup_magnetic_list(current).split('\n')
+        selection = SelectionBox(self.root, msg_list, [], 'Select Magnetic SpaceGroup', False).show()
+        if len(selection) > 0:
+            new_sg = selection[0].split()[3]
+            self.Symmetry.load_magnetic_spacegroup(new_sg)
+            self.fun_set()
+
     def fun_symuvw(self, event=None):
         """create symmetric uvw position"""
         self.update()
@@ -863,10 +975,6 @@ class PropertiesGui:
                         activebackground=btn_active)
         var.pack(side=tk.LEFT)
 
-        var = tk.Button(line, text='X-Ray\nScattering Factor', font=BF, command=self.fun_xrscat, bg=btn,
-                        activebackground=btn_active)
-        var.pack(side=tk.LEFT)
-
         var = tk.Button(line, text='Magnetic\nForm Factor', font=BF, command=self.fun_magff, bg=btn,
                         activebackground=btn_active)
         var.pack(side=tk.LEFT)
@@ -875,6 +983,10 @@ class PropertiesGui:
                         activebackground=btn_active)
         var.pack(side=tk.LEFT)
 
+        var = tk.Button(line, text='X-Ray\nScattering Factor', font=BF, command=self.fun_xsf, bg=btn,
+                        activebackground=btn_active)
+        var.pack(side=tk.LEFT)
+
     def fun_frac(self):
         """Atomic Fraction"""
         z = self.zfraction.get()
@@ -941,15 +1053,9 @@ class PropertiesGui:
         elements = elements.replace(',', ' ')
         elelist = elements.split()
         out = fc.print_atom_properties(elelist)
-        StringViewer(out, 'Element Properties')
-
-    def fun_xrscat(self):
-        """Properties button"""
-        elements = self.atoms.get()
-        elements = elements.replace(',', ' ')
-        elelist = elements.split()
-        fp.plot_xray_scattering_factor(elelist)
-        plt.show()
+        width = 12 + 12*len(elelist)
+        if width > 120: width=120
+        StringViewer(out, 'Element Properties', width)
 
     def fun_magff(self):
         """Properties button"""
@@ -967,6 +1073,15 @@ class PropertiesGui:
         fp.plot_xray_attenuation(elelist)
         plt.show()
 
+    def fun_xsf(self):
+        """Properties button"""
+        elements = self.atoms.get()
+        elements = elements.replace(',', ' ')
+        elelist = elements.split()
+        for el in elelist:
+            fp.plot_xray_scattering_factor(el)
+        plt.show()
+
 
 class SuperstructureGui:
     """
